"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const container_1 = require("@luodexun/container");
const pg_promise_1 = __importDefault(require("pg-promise"));
const query_executor_1 = require("./sql/query-executor");
const utils_1 = require("./utils");
class PostgresConnection {
    constructor(options) {
        this.options = options;
        this.logger = container_1.app.resolvePlugin("logger");
    }
    async make() {
        if (this.db) {
            throw new Error("Database connection already initialised");
        }
        this.logger.debug("Connecting to database");
        this.cache = new Map();
        try {
            await this.connect();
            this.registerQueryExecutor();
            this.logger.info(`Connected to database:${this.options.connection.host}:${this.options.connection.port}`);
            return this;
        }
        catch (error) {
            container_1.app.forceExit("Unable to connect to the database!", error);
        }
        return undefined;
    }
    async connect() {
        const options = this.options;
        const pgp = pg_promise_1.default({
            ...options.initialization,
            ...{
                error: async (error, context) => {
                    // https://www.postgresql.org/docs/11/errcodes-appendix.html
                    // Class 53 — Insufficient Resources
                    // Class 57 — Operator Intervention
                    // Class 58 — System Error (errors external to PostgreSQL itself)
                    console.log(error);
                },
                receive(data) {
                    utils_1.camelizeColumns(pgp, data);
                }
            },
        });
        this.pgp = pgp;
        this.db = this.pgp(this.options.connection);
    }
    async disconnect() {
        this.logger.debug("Disconnecting from database");
        try {
            this.cache.clear();
        }
        catch (error) {
            this.logger.warn("Issue in commiting blocks, database might be corrupted");
            this.logger.warn(error.message);
        }
        this.pgp.end();
        this.logger.debug("Disconnected from database");
    }
    registerQueryExecutor() {
        this.query = new query_executor_1.QueryExecutor(this);
    }
}
exports.PostgresConnection = PostgresConnection;
//# sourceMappingURL=postgres-connection.js.map